@@ -4,7 +4,7 @@ from __future__ import print_function
 """ binio.py
 Module that implements a simple class for reading and writing structured binary data
 
-Version: 1.0.1b
+Version: 1.1.0b
 Author:  Alejandro López Correa
 Contact: alc@spika.net
 URL:     http://spika.net/py/binio/
@@ -19,7 +19,10 @@ import struct
 from collections import namedtuple
 import string
 
+# astr and ustr are needed because of differences between python 2 and 3 regarding strings and byte buffers
+# astr: constructor for "ascii string"
 astr = lambda : str()
+# ustr: constructor for "unicode string"
 try:
     unicode()
     ustr = lambda : unicode()
@@ -27,6 +30,14 @@ except:
     ustr = lambda : str()        # python 3
 
 VARIABLE_SIZE = "variable size"
+
+BYTE_ORDER_NATIVE = '='
+BYTE_ORDER_LITTLE_ENDIAN = '<'
+BYTE_ORDER_BIG_ENDIAN = '>'
+BYTE_ORDER_NETWORK = '!'
+
+BYTE_ORDER_LIST = [BYTE_ORDER_NATIVE, BYTE_ORDER_LITTLE_ENDIAN, BYTE_ORDER_BIG_ENDIAN, BYTE_ORDER_NETWORK]
+
 class EndOfFileError( EOFError ):
     pass
     
@@ -97,6 +108,14 @@ def _fread( f, n ):
 def _fwrite( f, x ):
     f.write( x )
 
+class Struct( object ):
+    def __init__( self, model ):
+        self.__model = model
+    def get_size_in_bytes( self ):
+        return self.__model.get_size_in_bytes( self.__dict__ )
+    def clear( self ):
+        self.__model.clear( self.__dict__ )
+
 class BinIO( object ):
     """ objectDef format:
     
@@ -153,15 +172,16 @@ class BinIO( object ):
         self.__byteSize = byteCount
         self.clear()
         
-    def get_size_in_bytes( self ):
+    def get_size_in_bytes( self, obj=None ):
         """ returns byte count of this struct or VARIABLE_SIZE """
         if self.__byteSize == VARIABLE_SIZE:
+            this = obj if obj else self.__dict__
             bcount = 0
             for item in self.__items:
                 if type(item.n) == int:
                     n = item.n
                 else:
-                    n = eval( item.n, self.__dict__ )
+                    n = eval( item.n, dict(), this )
                     
                 fmt = '%i%s' % (n,item.ts)
                 bcount += n*item.tssz
@@ -169,46 +189,81 @@ class BinIO( object ):
             bcount = self.__byteSize
         return bcount
         
-    def clear( self ):
+    def clear( self, obj=None ):
         """ initializes all items to default value """
+        this = obj if obj else self.__dict__
         for item in self.__items:
-            setattr( self, item.name, _C_TO_DEFVAL[item.tc] )
+            this[item.name] = _C_TO_DEFVAL[item.tc]
+            
+    def read_struct( self, f, byteOrder=BYTE_ORDER_NATIVE ):
+        x = Struct( self )
+        self._read( f, byteOrder, x.__dict__ )
+        return x
 
-    def read( self, f, byteOrder='=' ):
+    def read_dict( self, f, byteOrder=BYTE_ORDER_NATIVE ):
+        x = dict()
+        self._read( f, byteOrder, x )
+        return x
+
+    def read( self, f, byteOrder=BYTE_ORDER_NATIVE ):
         """ read data from f with given byteOrder
             returns number of bytes read
         """
-        assert byteOrder in ('=','<','>')
-        self.clear()
+        self._read( f, byteOrder, self.__dict__ )
+                
+    def write_struct( self, f, x, byteOrder=BYTE_ORDER_NATIVE ):
+        """ write data from struct x to file f with given byte order
+            no data is written until all values have been correctly packaged
+            returns number of bytes written
+        """
+        return self._write( f, byteOrder, x.__dict__ )
+
+    def write_dict( self, f, x, byteOrder=BYTE_ORDER_NATIVE ):
+        """ write data from dict x to file f with given byte order
+            no data is written until all values have been correctly packaged
+            returns number of bytes written
+        """
+        return self._write( f, byteOrder, x )
+
+    def write( self, f, byteOrder=BYTE_ORDER_NATIVE ):
+        """ write data to file f with given byte order
+            no data is written until all values have been correctly packaged
+            returns number of bytes written
+        """
+        return self._write( f, byteOrder, self.__dict__ )
+
+    def _read( self, f, byteOrder, dst ):
+        assert byteOrder in BYTE_ORDER_LIST
+        self.clear( dst )
         bytesRead = 0
         for item in self.__items:
             if type(item.n) == int:
                 n = item.n
             else:
-                n = eval( item.n, self.__dict__ )
+                n = eval( item.n, dict(), dst )
                 
             fmt = '%i%s' % (n,item.ts)
             bcount = n*item.tssz
             values = struct.unpack( byteOrder+fmt, _fread( f, bcount ) )
             bytesRead += bcount
             
-            setattr( self, item.name, self._process_read( values, item.tc, n ) )
+            dst[item.name] = self._process_read( values, item.tc, n )
         return bytesRead
-                
-    def write( self, f, byteOrder='=' ):
+
+    def _write( self, f, byteOrder, src ):
         """ write data to f with given byteOrder
             no data is written until all values have been correctly packaged
             returns number of bytes written
         """
-        assert byteOrder in ('=','<','>')
+        assert byteOrder in BYTE_ORDER_LIST
         toWrite = b''
         for item in self.__items:
             if type(item.n) == int:
                 n = item.n
             else:
-                n = eval( item.n, self.__dict__ )
+                n = eval( item.n, dict(), src )
 
-            values = self._process_write( getattr( self, item.name ), item.tc, n )
+            values = self._process_write( src[item.name], item.tc, n )
             if values:
                 try:
                     fmt = '%i%s' % (n,item.ts)
@@ -219,7 +274,7 @@ class BinIO( object ):
 
         _fwrite( f, toWrite )
         return len(toWrite)
-            
+
     @staticmethod
     def __validate_identifier( ident ):
         if ident[0] not in string.ascii_letters:
@@ -324,14 +379,15 @@ def test():
             1 : char            : item9
     """ )
     
-    def show_values():
-        for i in range(1,10):
+    NItem = 9
+    def show_values( obj ):
+        for i in range(1,NItem+1):
             x = 'item%i'%i
-            v = getattr( testFile, x )
+            v = getattr( obj, x )
             print( '%s %16s %s' % (x, type(v), v) )
     
     print( 'default values' )
-    show_values()
+    show_values( testFile )
     print( 'size in bytes:', testFile.get_size_in_bytes() )
 
     print( '\nwrite' )
@@ -350,7 +406,7 @@ def test():
     testFile.item8 = b'0123456789abcdef'
     testFile.item9 = 'z'
     
-    show_values()
+    show_values( testFile )
     print( 'size in bytes:', testFile.get_size_in_bytes() )
     
     n = testFile.write( open( 'test.binio', 'wb' ) )
@@ -359,8 +415,52 @@ def test():
     print( '\nread' )
     n = testFile.read( open( 'test.binio', 'rb' ) )
     print( 'bytes read:', n )
-    show_values()
+    show_values( testFile )
+
+    testFile2 = new( """
+            1 : uint8           : item1
+            1 : uint32          : item2
+            8 : uint32          : item3
+            1 : int32           : n4
+         n4*2 : string@utf16    : item4
+            1 : int32           : n5
+           n5 : string@utf8     : item5
+            1 : int32           : n6
+           n6 : string          : item6
+            1 : int32           : n7
+         n7*3 : double          : item7
+           16 : byte            : item8
+            1 : char            : item9
+    """ )
     
+    print( '\nread/write into/from separate objects check' )
+    obj = testFile2.read_struct( open( 'test.binio', 'rb' ) )
+    testFile2.write_struct( open( 'test.binio', 'wb' ), obj )
+    obj2 = testFile2.read_struct( open( 'test.binio', 'rb' ) )
+    for i in range(1,NItem+1):
+        x = 'item%i'%i
+        if getattr( obj, x ) != getattr( testFile, x ):
+            print( '(%s): error type 1' % x )
+        if getattr( obj, x ) != getattr( obj2, x ):
+            print( '(%s): error type 2' % x )
+    assert obj.get_size_in_bytes() == testFile.get_size_in_bytes()
+    assert obj.get_size_in_bytes() == obj2.get_size_in_bytes()   
+    print( 'ok' )
+
+    print( '\nread/write into/from dicts' )
+    obj = testFile2.read_dict( open( 'test.binio', 'rb' ) )
+    testFile2.write_dict( open( 'test.binio', 'wb' ), obj )
+    obj2 = testFile2.read_dict( open( 'test.binio', 'rb' ) )
+    for i in range(1,NItem+1):
+        x = 'item%i'%i
+        if obj[x] != getattr( testFile, x ):
+            print( '(%s): error type 1' % x )
+        if obj[x] != obj2[x]:
+            print( '(%s): error type 2' % x )
+    assert testFile2.get_size_in_bytes( obj ) == testFile.get_size_in_bytes()
+    assert testFile2.get_size_in_bytes( obj ) == testFile2.get_size_in_bytes( obj2 )
+    print( 'ok' )
+
 if __name__ == '__main__':
     test()
 
